function [VI] = Gen_Impulsive_Noise(m,n,Nb,Ns,K,Pin,Pcr,A,B,Beta,Op_IN_Tipe,SNR)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Funao que simula o efeito de um rido impulsivo nas amostras coletadas %%%%%%%%
%%%%%% Entradas : (m,n,Nb,Ns,K,Pin,Pcr,SNR) Retorna: VI_mxn
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if Op_IN_Tipe == 1
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Ruido impulsivo Versao 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	Sigma_i = sqrt( K / (10^(SNR/10)));
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%% Preenche Zmxn com 1 em algumas posioes %%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	Z = zeros(m,n);
	for u = 1 : m
				
		g = zeros(1,n);	
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%%%%%% Intervalos das rajadas %%%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		t(2 * Nb) = 0;
		%Primeiro intervalo ti1, tf1
		t(1) = randi([0 (floor(n /Nb))]);
		t(2) = t(1) + Ns - 1;
		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%%% Se Tiver mais de 1 ruido impulsivo em um sensoriamento %%%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		if Nb > 1 
			for r = 1 : Nb -1		
				t(2*r + 1) = t(2*r) + randi([0 (floor(n /Nb))]) + Ns;
				t(2*r + 2) = t(2*r + 1) + Ns - 1;
			end 
		end
			
		z = t;
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%%% Analisa a posicao do 1 se passou de n volta na primeira %%%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		for r = 0 : Nb -1		
			for w = z(2*r+1) : z(2*r +2)			
				g(mod(w,n)+1)= 1;
			end			
		end
		
		Z(u,:)= g;	
		
	end
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%% Analisa a porcentagem dos rdios afetados %%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	for i = 1 : m
		aux_bino(i) = binornd(1,Pcr);
	end
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%% Elimina o ruido dos rdios 0 aux_bino %%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	G = diag(aux_bino) * Z;
	
	N = zeros(m,n);
	
	N= randn(m,n) +  1i* randn(m,n);                
	
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%% Gera matris de ruido nas posioes indicadas por G %%%	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	V2 = N.*G;
	
	for i = 1 : m 
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%%% Calcula energia do ruido Impusivo gerado por rdio %%%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		Pvi =  1/n * sum(abs(V2(i,:)).^2);
		
		if Pvi == 0
		VI(i,:) =  V2(i,:) * 0;
		else
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%%%% Normaliza e aplica fator K %%%%%%%%%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		VI(i,:) =  V2(i,:)/ sqrt(Pvi)* Sigma_i;
		end	
		
	end
else
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Ruido impulsivo Versao 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
	Sigma_i = sqrt( K / (10^(SNR/10)));
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Ruido impulsivo Versao 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%% Verify which CR will be affected%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	Afected_CR = binornd(1,Pcr,1,m);
	
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%%%%% Main Process of Generation %%%%%%%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% Generate Termal noise in all samples
	N = zeros(m,n) + 1i * zeros(m,n);
	
	for j = 1 : m
	
		% Calculate the first sample position of the RI.
		position_imp_noise = round(exprnd(Beta));
		
		for i = 1 : n 
		
			if i == position_imp_noise
				% Generates normal dist with A mean e deveation B
				X = A + B*randn(1);
				% Generates a lognomal amplitude of impulsive noise
				x = 10 .^(X/20);
				%x = lognrnd(A,B);
				% Generates the phase theta with uniform distrituition [0 2pi]
				theta = unifrnd(0,2*pi);
				% In phase and quadrature components of impulsive noise
				N(j,i) = x .*cos(theta) + 1i* x.*sin(theta);
				% Generates the next position for the impulsive noise
				position_imp_noise = i + round(exprnd(Beta));
					
			end
		
		end
	end;
	
	% Eliminates impulsive noise of unaffected radios
	V2 = diag(Afected_CR) * N;
	
	
	for i = 1 : m 
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%%% Calcula energia do ruido Impusivo gerado por radio %%%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		Pvi =  1/n * sum(abs(V2(i,:)).^2);
		
		if Pvi == 0
		VI(i,:) =  V2(i,:) * 0;
		else
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%%%% Normaliza e aplica fator K %%%%%%%%%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		VI(i,:) =  V2(i,:)/ sqrt(Pvi)* Sigma_i;
		end	
		
	end
end;













































